namespace newCalc; 
using display; 
using System.Collections.Generic;

public class CalcInvoker// invoker  
{
    public double DisplayValue {get; private set; }
    public Stack<ICommand> History { get; } = new Stack<ICommand>();

    public CalcInvoker() 
    {
        DisplayValue = 0; 
    }
    public void executeCommand(ICommand command) //tager en command og executer den og gemmer den på stacken så man kan undo den senere
    {
        DisplayValue = command.execute(DisplayValue); 
        History.Push(command); 

    }
    public void undoCommand() // tager den sidste command på stacken og undo'er den 
    {
        if(History.Count > 0)
        {
            var lastCommand = History.Pop();
            DisplayValue = lastCommand.undo(DisplayValue); 
        }

    }
}
public abstract class ICommand  //hoved klassen som alle vores commands arver fra
{
    protected Calc _calculator; 
    public ICommand(Calc calculator)
    {
        _calculator = calculator; 
    }
    public abstract double execute(double currentDisplayValue); // hver command skal implementere denne + undo
    public abstract double undo(double currentDisplayValue); 
}

public class AddCommand : ICommand // En command 
{
    private readonly double _valueToAdd; 
    
    public AddCommand(double valueToAdd, Calc calculator) : base (calculator)
    {
        _valueToAdd = valueToAdd;
        _calculator = calculator;  
    }

    public override double execute(double currentDisplayValue )
    {
        return _calculator.Add(_valueToAdd);
    }

    public override double undo(double currentDisplayValue)
    {
        return _calculator.Subtract(_valueToAdd); 
    }
}
public class subtractCommand : ICommand
{
    private readonly double _valueTosubtract; 
    public subtractCommand(double valueTosubtract, Calc calculator) : base (calculator)
    {
        _valueTosubtract = valueTosubtract; 
        _calculator = calculator;
    }

    public override double execute(double currentDisplayValue )
    {
        return _calculator.Subtract(_valueTosubtract);
    }

    public override double undo(double currentDisplayValue)
    {
        return _calculator.Add(_valueTosubtract);
    }

}
public class MultiplyCommand : ICommand
{
    private readonly double _valueToMultiply; 
    public MultiplyCommand(double valueToMultiply, Calc calculator) : base (calculator)
    {
        _valueToMultiply = valueToMultiply; 
        _calculator = calculator;
    }

    public override double execute(double currentDisplayValue )
    {
        return _calculator.Multiply(_valueToMultiply);
    }

    public override double undo(double currentDisplayValue)
    {
        return _calculator.Divide(_valueToMultiply);
    }
}

public class DivideCommand : ICommand
{
    private readonly double _valueToDivide; 
    public DivideCommand(double valueToDivide, Calc calculator) : base (calculator)
    {
        _valueToDivide = valueToDivide; 
        _calculator = calculator;
    }

    public override double execute(double currentDisplayValue )
    {
        return _calculator.Divide(_valueToDivide);
    }

    public override double undo(double currentDisplayValue)
    {
        return _calculator.Multiply(_valueToDivide); 
    }

}

